systemc: Ignore process control functions on terminated processes.

These functions can descend to the children of a terminated process,
but should have no effect on that process itself.

Change-Id: I6e4bdec8c492dd03d05bc1397aa080e8a51397c1
Reviewed-on: https://gem5-review.googlesource.com/c/14917
Reviewed-by: Matthias Jung <jungma@eit.uni-kl.de>
Maintainer: Gabe Black <gabeblack@google.com>
diff --git a/src/systemc/core/process.cc b/src/systemc/core/process.cc
index 193056f..aaa42e8 100644
--- a/src/systemc/core/process.cc
+++ b/src/systemc/core/process.cc
@@ -81,7 +81,7 @@
     if (inc_kids)
         forEachKid([](Process *p) { p->suspend(true); });
 
-    if (!_suspended) {
+    if (!_suspended && !_terminated) {
         _suspended = true;
         _suspendedReady = scheduler.suspend(this);
 
@@ -102,7 +102,7 @@
     if (inc_kids)
         forEachKid([](Process *p) { p->resume(true); });
 
-    if (_suspended) {
+    if (_suspended && !_terminated) {
         _suspended = false;
         if (_suspendedReady)
             scheduler.resume(this);
@@ -124,7 +124,8 @@
                 message.c_str());
     }
 
-    _disabled = true;
+    if (!_terminated)
+        _disabled = true;
 }
 
 void
@@ -134,7 +135,8 @@
     if (inc_kids)
         forEachKid([](Process *p) { p->enable(true); });
 
-    _disabled = false;
+    if (!_terminated)
+        _disabled = false;
 }
 
 void
@@ -149,8 +151,8 @@
     if (inc_kids)
         forEachKid([](Process *p) { p->kill(true); });
 
-    // If we're in the middle of unwinding, ignore the kill request.
-    if (_isUnwinding)
+    // If we're unwinding or terminated, ignore the kill request.
+    if (_isUnwinding || _terminated)
         return;
 
     // Update our state.
@@ -177,8 +179,8 @@
     if (inc_kids)
         forEachKid([](Process *p) { p->reset(true); });
 
-    // If we're in the middle of unwinding, ignore the reset request.
-    if (_isUnwinding)
+    // If we're already unwinding or terminated, ignore the reset request.
+    if (_isUnwinding || _terminated)
         return;
 
     // Clear suspended ready since we're about to run regardless.